module emsx_top(
    // Clock, Reset ports
    pClk21m,
    pExtClk,
    pCpuClk,
    // MSX cartridge slot ports
    pSltClk,
    pSltRst_n,
    pSltSltsl_n,
    pSltSlts2_n,
    pSltIorq_n,
    pSltRd_n,
    pSltWr_n,
    pSltAdr,
    pSltDat,
    pSltBdir_n, // Bus direction (not used in master mode)
    //
    pSltCs1_n,
    pSltCs2_n,
    pSltCs12_n,
    pSltRfsh_n,
    pSltWait_n,
    pSltInt_n,
    pSltM1_n,
    pSltMerq_n,
    //
    pSltRsv5,
    pSltRsv16,
    pSltSw1,
    pSltSw2,
    // SD-RAM ports
    pMemClk,
    pMemCke,
    pMemCs_n,
    pMemRas_n,
    pMemCas_n,
    pMemWe_n,
    pMemUdq,
    pMemLdq,
    pMemBa1,
    pMemBa0,
    pMemAdr,
    pMemDat,
    // PS/2 keyboard ports
    pPs2Clk,
    pPs2Dat,
    // Joystick ports (Port_A, Port_B)
    pJoyA,
    pStrA,
    pJoyB,
    pStrB,
    // SD/MMC slot ports
    pSd_Ck,
    pSd_Cm,
    pSd_Dt,
    // DIP switch, Lamp ports
    pDip,
    pLed,
    pLedPwr,
    // Video, Audio/CMT ports
    pDac_VR,
    pDac_VG,
    pDac_VB,
    pDac_SL,
    pDac_SR,
    pVideoHS_n,
    pVideoVS_n,
    pVideoClk,
    pVideoDat,
    // Reserved ports (USB)
    pUsbP1,
    pUsbN1,
    pUsbP2,
    pUsbN2,
    // Reserved ports
    pIopRsv14,
    pIopRsv15,
    pIopRsv16,
    pIopRsv17,
    pIopRsv18,
    pIopRsv19,
    pIopRsv20,
    pIopRsv21
  );
  
  // CLK = Clock input
  // PLL = PLL Output
  // IPU = Internal Pull Up
  // EPU = External Pull Up
  
    // Clock, Reset ports
    input           pClk21m;    // CLK : Master Clock(21.48MHz) from XTAL
    input           pExtClk;    // CLK : same as pClk21m
    output          pCpuClk;    // PLL : Clock out. Connected pSltClk
                                //       Optionanl Function PLL2_OUTp

    // MSX cartridge slot ports
    input           pSltClk;    // CLK : Clock in. Connected pCpuClk
    input           pSltRst_n;  // EPU : RESET from Reset-IC(M519538FP)
    inout           pSltSltsl_n;// -   : Select cartrige 1
    inout           pSltSlts2_n;// -   : Select cartrige 2
    inout           pSltIorq_n; // IPU : 
    inout           pSltRd_n;   // IPU : 
    inout           pSltWr_n;   // IPU : 
    inout   [15: 0] pSltAdr;    // -   :
    inout   [ 7: 0] pSltDat;    // IPU : 
    output          pSltBdir_n; // IPU :   Bus direction (not used in master mode)

    inout           pSltCs1_n;  // -   :
    inout           pSltCs2_n;  // -   :
    inout           pSltCs12_n; // -   :
    inout           pSltRfsh_n; // IPU : 
    inout           pSltWait_n; // EPU+IPU : 
    inout           pSltInt_n;  // EPU+IPU : 
    inout           pSltM1_n;   // IPU : 
    inout           pSltMerq_n; // IPU : 

    output          pSltRsv5;   // -   : Reserved
    output          pSltRsv16;  // EPU :  Reserved
    inout           pSltSw1;    // -   : Reserved
    inout           pSltSw2;    // -   : Reserved

    // SD-RAM ports
    output          pMemClk;    // -   :  SD-RAM Clock
    output          pMemCke;    // -   :  SD-RAM Clock enable
    output          pMemCs_n;   // -   :  SD-RAM Chip select
    output          pMemRas_n;  // -   :  SD-RAM Row/RAS
    output          pMemCas_n;  // -   :  SD-RAM /CAS
    output          pMemWe_n;   // -   :  SD-RAM /WE
    output          pMemUdq;    // -   :  SD-RAM UDQM
    output          pMemLdq;    // -   :  SD-RAM LDQM
    output          pMemBa1;    // -   :  SD-RAM Bank select address 1
    output          pMemBa0;    // -   :  SD-RAM Bank select address 0
    output  [12: 0] pMemAdr;    // -   :  SD-RAM Address
    inout   [15: 0] pMemDat;    // IPU :   SD-RAM Data

    // PS/2 keyboard ports
    inout           pPs2Clk;    // EPU+IPU : 
    inout           pPs2Dat;    // EPU+IPU : 

    // Joystick ports (Port_A, Port_B)
    inout   [ 5: 0] pJoyA;      // EPU+IPU : 
    output          pStrA;      // IPU :      Joystick Common output
    inout   [ 5: 0] pJoyB;      // EPU+IPU : 
    output          pStrB;      // IPU :      Joystick Common output

    // SD/MMC slot ports
    output          pSd_Ck;     // EPU+IPU :   pin 5(CLK)
    output          pSd_Cm;     // EPU+IPU :   pin 2(CMD)
    inout   [ 3: 0] pSd_Dt;     // IPU : pin 1(D3), 9(D2), 8(D1), 7(D0)
                                //             D3,D2,D1 use for SD only

    // DIP switch, Lamp ports
    input   [ 7: 0] pDip;       // IPU :   DIP Switch 0=ON,  1=OFF
    output  [ 7: 0] pLed;       // IPU :   LED 0=OFF, 1=ON(green)
    output          pLedPwr;    // IPU :   LED 0=OFF, 1=ON(red) ...Power & SD/MMC access lamp

    // Video, Audio/CMT ports
    inout   [ 5: 0] pDac_VR;    // Ladder DAC : RGB_Red / Svideo_C
    inout   [ 5: 0] pDac_VG;    // Ladder DAC : RGB_Grn / Svideo_Y
    inout   [ 5: 0] pDac_VB;    // Ladder DAC : RGB_Blu / CompositeVideo
    output  [ 5: 0] pDac_SL;    // Ladder DAC : Sound-L
    inout   [ 5: 0] pDac_SR;    // Ladder DAC : Sound-R / CMT

    output  pVideoHS_n;         // IPU :  Csync(RGB15K), HSync(VGA31K)
    output  pVideoVS_n;         // IPU :  Audio(RGB15K), VSync(VGA31K)

    output  pVideoClk;          // IPU :  (Reserved)
    output  pVideoDat;          // IPU :  (Reserved)

    // Reserved ports (USB)
    inout   pUsbP1;             // EPU : 
    inout   pUsbN1;             // EPU : 
    inout   pUsbP2;             // EPU : 
    inout   pUsbN2;             // EPU : 

    // Reserved ports
    input   pIopRsv14;  // IPU : 
    input   pIopRsv15;  // IPU : 
    input   pIopRsv16;  // IPU : 
    input   pIopRsv17;  // IPU : 
    input   pIopRsv18;  // IPU : 
    input   pIopRsv19;  // IPU : 
    input   pIopRsv20;  // IPU : 
    input   pIopRsv21;  // IPU : 

//====================================================
// wire/regs
//====================================================
    wire        clk;
    wire        rst;
    
    wire    [15: 0] pSltAdr;
    wire    [ 7: 0] pSltDat;
    wire    [12: 0] pMemAdr;
    wire    [15: 0] pMemDat;
    wire    [ 5: 0] pJoyA;
    wire    [ 5: 0] pJoyB;
    wire    [ 3: 0] pSd_Dt;
    wire    [ 7: 0] pLed;
    wire    [ 5: 0] pDac_VR;
    wire    [ 5: 0] pDac_VG;
    wire    [ 5: 0] pDac_VB;
    wire    [ 5: 0] pDac_SL;
    wire    [ 5: 0] pDac_SR;
    
    reg [23: 0] cnt;

//====================================================
// logic
//====================================================
    assign rst = pSltRst_n;
    
    assign clk = pClk21m;
    assign pCpuClk = clk;

    // MSX cartridge slot ports
    assign pSltSltsl_n = 1'bz;
    assign pSltSlts2_n = 1'bz;
    assign pSltIorq_n  = 1'bz;
    assign pSltRd_n    = 1'bz;
    assign pSltWr_n    = 1'bz;
    assign pSltAdr     = 16'hzzzz;
    assign pSltDat     = 8'hzz;
    assign pSltBdir_n  = 1'b0;

    assign pSltCs1_n  = 1'bz;
    assign pSltCs2_n  = 1'bz;
    assign pSltCs12_n = 1'bz;
    assign pSltRfsh_n = 1'bz;
    assign pSltWait_n = 1'bz;
    assign pSltInt_n  = 1'bz;
    assign pSltM1_n   = 1'bz;
    assign pSltMerq_n = 1'bz;

    assign pSltRsv5  = 1'b0;
    assign pSltRsv16 = 1'b0;
    assign pSltSw1   = 1'bz;
    assign pSltSw2   = 1'bz;

    // SD-RAM ports
    assign pMemClk   = 1'b1;
    assign pMemCke   = 1'b0;
    assign pMemCs_n  = 1'b1;
    assign pMemRas_n = 1'b1;
    assign pMemCas_n = 1'b1;
    assign pMemWe_n  = 1'b1;
    assign pMemUdq   = 1'b0;
    assign pMemLdq   = 1'b0;
    assign pMemBa1   = 1'b0;
    assign pMemBa0   = 1'b0;
    assign pMemAdr   = 13'h0000;
    assign pMemDat   = 16'hzzzz;

    // PS/2 keyboard ports
    assign pPs2Clk = 1'bz;
    assign pPs2Dat = 1'bz;

    // Joystick ports (Port_A, Port_B)
    assign pJoyA = 6'hzz;
    assign pStrA = 1'b0;
    assign pJoyB = 6'hzz;
    assign pStrB = 1'b0;

    // SD/MMC slot ports
    assign pSd_Ck = 1'b1;
    assign pSd_Cm = 1'b1;
    assign pSd_Dt = 4'hF;

    // DIP switch, Lamp ports
    assign pLed = pDip;

    always @(posedge clk)
      if(!rst)
        cnt <= 24'h0000;
      else
        cnt <= cnt + 24'h0001;

    assign pLedPwr = cnt[23];

    // Video, Audio/CMT ports
    assign pDac_VR = 6'h00;    // Ladder DAC : RGB_Red / Svideo_C
    assign pDac_VG = 6'h00;    // Ladder DAC : RGB_Grn / Svideo_Y
    assign pDac_VB = 6'h00;    // Ladder DAC : RGB_Blu / CompositeVideo
    assign pDac_SL = 6'h00;    // Ladder DAC : Sound-L
    assign pDac_SR = 6'h00;    // Ladder DAC : Sound-R / CMT

    assign pVideoHS_n = 1'b1;
    assign pVideoVS_n = 1'b1;

    assign pVideoClk = 1'b1;
    assign pVideoDat = 1'b1;

    // Reserved ports (USB)
    assign pUsbP1 = 1'bz;
    assign pUsbN1 = 1'bz;
    assign pUsbP2 = 1'bz;
    assign pUsbN2 = 1'bz;

endmodule
